@@ -1,8 +1,8 @@
 import os
-import sys
+import localstack_client.config
 
 # LocalStack version
-VERSION = '0.7.4'
+VERSION = '0.10.2'
 
 # default AWS region
 if 'DEFAULT_REGION' not in os.environ:
@@ -15,23 +15,6 @@ REGION_LOCAL = 'local'
 # dev environment
 ENV_DEV = 'dev'
 
-# infra service ports (counting up from 4567)
-DEFAULT_PORT_APIGATEWAY = 4567
-DEFAULT_PORT_KINESIS = 4568
-DEFAULT_PORT_DYNAMODB = 4569
-DEFAULT_PORT_DYNAMODBSTREAMS = 4570
-DEFAULT_PORT_ELASTICSEARCH = 4571
-DEFAULT_PORT_S3 = 4572
-DEFAULT_PORT_FIREHOSE = 4573
-DEFAULT_PORT_LAMBDA = 4574
-DEFAULT_PORT_SNS = 4575
-DEFAULT_PORT_SQS = 4576
-DEFAULT_PORT_REDSHIFT = 4577
-DEFAULT_PORT_ES = 4578
-DEFAULT_PORT_SES = 4579
-DEFAULT_PORT_ROUTE53 = 4580
-DEFAULT_PORT_CLOUDFORMATION = 4581
-DEFAULT_PORT_CLOUDWATCH = 4582
 # backend service ports, for services that are behind a proxy (counting down from 4566)
 DEFAULT_PORT_APIGATEWAY_BACKEND = 4566
 DEFAULT_PORT_KINESIS_BACKEND = 4565
@@ -41,30 +24,18 @@ DEFAULT_PORT_SNS_BACKEND = 4562
 DEFAULT_PORT_SQS_BACKEND = 4561
 DEFAULT_PORT_ELASTICSEARCH_BACKEND = 4560
 DEFAULT_PORT_CLOUDFORMATION_BACKEND = 4559
+DEFAULT_PORT_STEPFUNCTIONS_BACKEND = 4558
+DEFAULT_PORT_IAM_BACKEND = 4557
 
 DEFAULT_PORT_WEB_UI = 8080
 
 LOCALHOST = 'localhost'
 
-# map of default service APIs and ports to be spun up
-DEFAULT_SERVICE_PORTS = {
-    'es': DEFAULT_PORT_ES,
-    'elasticsearch': DEFAULT_PORT_ELASTICSEARCH,
-    's3': DEFAULT_PORT_S3,
-    'sns': DEFAULT_PORT_SNS,
-    'sqs': DEFAULT_PORT_SQS,
-    'apigateway': DEFAULT_PORT_APIGATEWAY,
-    'dynamodb': DEFAULT_PORT_DYNAMODB,
-    'dynamodbstreams': DEFAULT_PORT_DYNAMODBSTREAMS,
-    'firehose': DEFAULT_PORT_FIREHOSE,
-    'lambda': DEFAULT_PORT_LAMBDA,
-    'kinesis': DEFAULT_PORT_KINESIS,
-    'redshift': DEFAULT_PORT_REDSHIFT,
-    'route53': DEFAULT_PORT_ROUTE53,
-    'ses': DEFAULT_PORT_SES,
-    'cloudformation': DEFAULT_PORT_CLOUDFORMATION,
-    'cloudwatch': DEFAULT_PORT_CLOUDWATCH
-}
+# version of the Maven dependency with Java utility code
+LOCALSTACK_MAVEN_VERSION = '0.1.22'
+
+# map of default service APIs and ports to be spun up (fetch map from localstack_client)
+DEFAULT_SERVICE_PORTS = localstack_client.config.get_service_ports()
 
 # host to bind to when starting the services
 BIND_HOST = '0.0.0.0'
@@ -88,6 +59,9 @@ PATH_USER_REQUEST = '_user_request_'
 # name of LocalStack Docker image
 DOCKER_IMAGE_NAME = 'localstack/localstack'
 
+# backdoor API path used to retrieve or update config variables
+CONFIG_UPDATE_PATH = '/?_config_'
+
 # environment variable name to tag local test runs
 ENV_INTERNAL_TEST_RUN = 'LOCALSTACK_INTERNAL_TEST_RUN'
 
@@ -95,13 +69,23 @@ ENV_INTERNAL_TEST_RUN = 'LOCALSTACK_INTERNAL_TEST_RUN'
 APPLICATION_AMZ_JSON_1_0 = 'application/x-amz-json-1.0'
 APPLICATION_AMZ_JSON_1_1 = 'application/x-amz-json-1.1'
 APPLICATION_JSON = 'application/json'
+APPLICATION_X_WWW_FORM_URLENCODED = 'application/x-www-form-urlencoded'
 
 # Lambda defaults
-LAMBDA_TEST_ROLE = "arn:aws:iam::%s:role/lambda-test-role" % TEST_AWS_ACCOUNT_ID
+LAMBDA_TEST_ROLE = 'arn:aws:iam::%s:role/lambda-test-role' % TEST_AWS_ACCOUNT_ID
 
 # installation constants
-ELASTICSEARCH_JAR_URL = 'https://artifacts.elastic.co/downloads/elasticsearch/elasticsearch-5.3.0.zip'
+ELASTICSEARCH_JAR_URL = 'https://artifacts.elastic.co/downloads/elasticsearch/elasticsearch-6.7.0.zip'
+# See https://docs.aws.amazon.com/ja_jp/elasticsearch-service/latest/developerguide/aes-supported-plugins.html
+ELASTICSEARCH_PLUGIN_LIST = ['analysis-icu', 'ingest-attachment', 'analysis-kuromoji',
+ 'mapper-murmur3', 'mapper-size', 'analysis-phonetic', 'analysis-smartcn', 'analysis-stempel', 'analysis-ukrainian']
 DYNAMODB_JAR_URL = 'https://s3-us-west-2.amazonaws.com/dynamodb-local/dynamodb_local_latest.zip'
+ELASTICMQ_JAR_URL = 'https://s3-eu-west-1.amazonaws.com/softwaremill-public/elasticmq-server-0.14.2.jar'
+STS_JAR_URL = 'https://repo1.maven.org/maven2/com/amazonaws/aws-java-sdk-sts/1.11.14/aws-java-sdk-sts-1.11.14.jar'
+STEPFUNCTIONS_ZIP_URL = 'https://s3.amazonaws.com/stepfunctionslocal/StepFunctionsLocal.zip'
 
 # API endpoint for analytics events
-API_ENDPOINT = 'https://api.localstack.cloud/v1'
+API_ENDPOINT = os.environ.get('API_ENDPOINT') or 'https://api.localstack.cloud/v1'
+
+# Hardcoded AWS account ID used by moto
+MOTO_ACCOUNT_ID = '123456789012'
