@@ -2,15 +2,20 @@ import re
 import os
 import json
 import logging
-import base64
-import datetime
+import socket
 import tempfile
-from localstack.utils.common import *
-from localstack.utils.aws.aws_models import *
+from localstack.utils.common import (short_uid, parallelize, is_port_open, new_tmp_file,
+    rm_rf, unzip, download, clean_cache, mktime, load_file, mkdir, run, md5)
+from localstack.utils.aws.aws_models import (ElasticSearch, S3Notification,
+    EventSource, DynamoDB, DynamoDBStream, FirehoseStream, S3Bucket, SqsQueue,
+    KinesisShard, KinesisStream, LambdaFunction)
 from localstack.utils.aws import aws_stack
-from localstack.constants import REGION_LOCAL, DEFAULT_REGION
+from localstack.utils.common import to_str
+from localstack.constants import DEFAULT_REGION
 from six import iteritems
 
+# TODO: CLI commands in this file need to be replaced with SDK calls!
+
 
 AWS_CACHE_TIMEOUT = 5  # 5 seconds
 AWS_LAMBDA_CODE_CACHE_TIMEOUT = 5 * 60  # 5 minutes
@@ -36,7 +41,16 @@ def run_cached(cmd, cache_duration_secs=None):
         'AWS_DEFAULT_REGION': os.environ.get('AWS_DEFAULT_REGION') or DEFAULT_REGION,
         'PYTHONWARNINGS': 'ignore:Unverified HTTPS request'
     })
-    return run(cmd, cache_duration_secs=cache_duration_secs, env_vars=env_vars)
+    tmp_file_path = new_tmp_file()
+    error = None
+    with open(tmp_file_path, 'w') as err_file:
+        try:
+            return run(cmd, cache_duration_secs=cache_duration_secs, env_vars=env_vars, stderr=err_file)
+        except Exception as e:
+            error = e
+    if error:
+        LOG.warning('Error running command: %s %s %s' % (cmd, error, load_file(tmp_file_path)))
+        raise error
 
 
 def run_aws_cmd(service, cmd_params, env=None, cache_duration_secs=None):
@@ -80,7 +94,7 @@ def aws_cmd(service, env):
     cmd = '{ test `which aws` || . .venv/bin/activate; }; aws'
     endpoint_url = None
     env = aws_stack.get_environment(env)
-    if env.region == REGION_LOCAL:
+    if aws_stack.is_local_env(env):
         endpoint_url = aws_stack.get_local_service_url(service)
     if endpoint_url:
         if endpoint_url.startswith('https://'):
@@ -108,7 +122,7 @@ def get_kinesis_streams(filter='.*', pool={}, env=None):
                 pool[arn] = stream
                 stream.shards = get_kinesis_shards(stream_details=details, env=env)
                 result.append(stream)
-    except socket.error as e:
+    except socket.error:
         pass
     return result
 
@@ -141,7 +155,7 @@ def get_sqs_queues(filter='.*', pool={}, env=None):
             if re.match(filter, name):
                 queue = SqsQueue(arn)
                 result.append(queue)
-    except socket.error as e:
+    except socket.error:
         pass
     return result
 
@@ -150,7 +164,7 @@ def get_sqs_queues(filter='.*', pool={}, env=None):
 def resolve_string_or_variable(string, code_map):
     if re.match(r'^["\'].*["\']$', string):
         return string.replace('"', '').replace("'", '')
-    LOG.warning("Variable resolution not implemented")
+    LOG.warning('Variable resolution not implemented')
     return None
 
 
@@ -218,14 +232,14 @@ def get_lambda_functions(filter='.*', details=False, pool={}, env=None):
                 try:
                     code_map = get_lambda_code(func_name, env=env)
                     f.targets = extract_endpoints(code_map, pool)
-                except Exception as e:
+                except Exception:
                     LOG.warning("Unable to get code for lambda '%s'" % func_name)
 
     try:
         out = cmd_lambda('list-functions', env)
         out = json.loads(out)
         parallelize(handle, out['Functions'])
-    except socket.error as e:
+    except socket.error:
         pass
     return result
 
@@ -247,7 +261,7 @@ def get_lambda_code(func_name, retries=1, cache_time=None, env=None):
     if MOCK_OBJ:
         return ''
     env = aws_stack.get_environment(env)
-    if cache_time is None and env.region != REGION_LOCAL:
+    if cache_time is None and not aws_stack.is_local_env(env):
         cache_time = AWS_LAMBDA_CODE_CACHE_TIMEOUT
     out = cmd_lambda('get-function --function-name %s' % func_name, env, cache_time)
     out = json.loads(out)
@@ -259,16 +273,17 @@ def get_lambda_code(func_name, retries=1, cache_time=None, env=None):
     try:
         mkdir(folder)
         if not os.path.isfile(archive):
-            download(loc, archive)
+            download(loc, archive, verify_ssl=False)
         if len(os.listdir(folder)) <= 1:
-            run("cd %s && unzip -o %s" % (folder, filename))
+            zip_path = os.path.join(folder, filename)
+            unzip(zip_path, folder)
     except Exception as e:
-        print("WARN: %s" % e)
+        print('WARN: %s' % e)
         rm_rf(archive)
         if retries > 0:
             return get_lambda_code(func_name, retries=retries - 1, cache_time=1, env=env)
         else:
-            print("WARNING: Unable to retrieve lambda code: %s" % e)
+            print('WARNING: Unable to retrieve lambda code: %s' % e)
 
     # traverse subdirectories and get script sources
     result = {}
@@ -284,6 +299,8 @@ def get_lambda_code(func_name, retries=1, cache_time=None, env=None):
     clean_cache(file_pattern=TMP_DOWNLOAD_FILE_PATTERN,
         last_clean_time=last_cache_cleanup_time,
         max_age=TMP_DOWNLOAD_CACHE_MAX_AGE)
+    # TODO: delete only if cache_time is over
+    rm_rf(folder)
 
     return result
 
@@ -301,11 +318,11 @@ def get_elasticsearch_domains(filter='.*', pool={}, env=None):
                 details = json.loads(details)['DomainStatus']
                 arn = details['ARN']
                 es = ElasticSearch(arn)
-                es.endpoint = details['Endpoint']
+                es.endpoint = details.get('Endpoint', 'n/a')
                 result.append(es)
                 pool[arn] = es
         parallelize(handle, out['DomainNames'])
-    except socket.error as e:
+    except socket.error:
         pass
 
     return result
@@ -329,7 +346,7 @@ def get_dynamo_dbs(filter='.*', pool={}, env=None):
                 result.append(db)
                 pool[arn] = db
         parallelize(handle, out['TableNames'])
-    except socket.error as e:
+    except socket.error:
         pass
     return result
 
@@ -346,7 +363,7 @@ def get_s3_buckets(filter='.*', pool={}, details=False, env=None):
             pool[arn] = bucket
             if details:
                 try:
-                    out = cmd_s3api('get-bucket-notification --bucket %s' % bucket_name, env=env)
+                    out = cmd_s3api('get-bucket-notification-configuration --bucket %s' % bucket_name, env=env)
                     if out:
                         out = json.loads(out)
                         if 'CloudFunctionConfiguration' in out:
@@ -356,13 +373,13 @@ def get_s3_buckets(filter='.*', pool={}, details=False, env=None):
                             n.target = func
                             bucket.notifications.append(n)
                 except Exception as e:
-                    print("WARNING: Unable to get details for bucket: %s" % e)
+                    print('WARNING: Unable to get details for bucket: %s' % e)
 
     try:
         out = cmd_s3api('list-buckets', env)
         out = json.loads(out)
         parallelize(handle, out['Buckets'])
-    except socket.error as e:
+    except socket.error:
         pass
     return result
 
@@ -381,10 +398,10 @@ def get_firehose_streams(filter='.*', pool={}, env=None):
                 s = FirehoseStream(arn)
                 for dest in details['Destinations']:
                     dest_s3 = dest['S3DestinationDescription']['BucketARN']
-                    bucket = func = EventSource.get(dest_s3, pool=pool)
+                    bucket = EventSource.get(dest_s3, pool=pool)
                     s.destinations.append(bucket)
                 result.append(s)
-    except socket.error as e:
+    except socket.error:
         pass
     return result
 
@@ -392,13 +409,12 @@ def get_firehose_streams(filter='.*', pool={}, env=None):
 def read_kinesis_iterator(shard_iterator, max_results=10, env=None):
     data = cmd_kinesis('get-records --shard-iterator %s --limit %s' %
         (shard_iterator, max_results), env, cache_duration_secs=0)
-    data = json.loads(data)
+    data = json.loads(to_str(data))
     result = data
     return result
 
 
 def get_kinesis_events(stream_name, shard_id, max_results=10, env=None):
-    timestamp = now() - KINESIS_RECENT_EVENTS_TIME_DIFF_SECS
     env = aws_stack.get_environment(env)
     records = aws_stack.kinesis_get_latest_records(stream_name, shard_id, count=max_results, env=env)
     for r in records:
