@@ -1,33 +1,45 @@
-from six.moves.BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
-import requests
 import os
 import sys
-import json
-import traceback
-import logging
 import ssl
+import json
+import socket
+import inspect
+import logging
+import traceback
+import click
+import requests
+from ssl import SSLError
 from flask_cors import CORS
 from requests.structures import CaseInsensitiveDict
 from requests.models import Response, Request
-from six import iteritems, string_types
+from six import iteritems
 from six.moves.socketserver import ThreadingMixIn
 from six.moves.urllib.parse import urlparse
-from localstack.config import DEFAULT_ENCODING, TMP_FOLDER, USE_SSL
-from localstack.utils.common import FuncThread, generate_ssl_cert
-from localstack.utils.compat import bytes_
+from six.moves.BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
+from localstack.config import TMP_FOLDER, USE_SSL, EXTRA_CORS_ALLOWED_HEADERS, EXTRA_CORS_EXPOSE_HEADERS
+from localstack.constants import ENV_INTERNAL_TEST_RUN
+from localstack.utils.common import FuncThread, generate_ssl_cert, to_bytes
 
 QUIET = False
 
 # path for test certificate
 SERVER_CERT_PEM_FILE = '%s/server.test.pem' % (TMP_FOLDER)
 
-# CORS settings
-CORS_ALLOWED_HEADERS = ('authorization', 'content-type', 'content-md5',
-    'x-amz-content-sha256', 'x-amz-date', 'x-amz-security-token', 'x-amz-user-agent')
+
+CORS_ALLOWED_HEADERS = ('authorization', 'content-type', 'content-md5', 'cache-control',
+    'x-amz-content-sha256', 'x-amz-date', 'x-amz-security-token', 'x-amz-user-agent',
+    'x-amz-acl', 'x-amz-version-id')
+if EXTRA_CORS_ALLOWED_HEADERS:
+    CORS_ALLOWED_HEADERS += tuple(EXTRA_CORS_ALLOWED_HEADERS.split(','))
+
 CORS_ALLOWED_METHODS = ('HEAD', 'GET', 'PUT', 'POST', 'DELETE', 'OPTIONS', 'PATCH')
 
+CORS_EXPOSE_HEADERS = ('x-amz-version-id',)
+if EXTRA_CORS_EXPOSE_HEADERS:
+    CORS_EXPOSE_HEADERS += tuple(EXTRA_CORS_EXPOSE_HEADERS.split(','))
+
 # set up logger
-LOGGER = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 class ThreadedHTTPServer(ThreadingMixIn, HTTPServer):
@@ -70,6 +82,9 @@ class ProxyListener(object):
 
 class GenericProxyHandler(BaseHTTPRequestHandler):
 
+    # List of `ProxyListener` instances that are enabled by default for all requests
+    DEFAULT_LISTENERS = []
+
     def __init__(self, request, client_address, server):
         self.request = request
         self.client_address = client_address
@@ -77,7 +92,10 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
         self.proxy = server.my_object
         self.data_bytes = None
         self.protocol_version = self.proxy.protocol_version
-        BaseHTTPRequestHandler.__init__(self, request, client_address, server)
+        try:
+            BaseHTTPRequestHandler.__init__(self, request, client_address, server)
+        except SSLError as e:
+            LOG.warning('SSL error when handling request: %s' % e)
 
     def parse_request(self):
         result = BaseHTTPRequestHandler.parse_request(self)
@@ -87,68 +105,99 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
             return result
         # Required fix for Python 2 (otherwise S3 uploads are hanging), based on the Python 3 code:
         # https://sourcecodebrowser.com/python3.2/3.2.3/http_2server_8py_source.html#l00332
-        expect = self.headers.get('Expect', "")
-        if (expect.lower() == "100-continue" and
-                self.protocol_version >= "HTTP/1.1" and
-                self.request_version >= "HTTP/1.1"):
+        expect = self.headers.get('Expect', '')
+        if (expect.lower() == '100-continue' and
+                self.protocol_version >= 'HTTP/1.1' and
+                self.request_version >= 'HTTP/1.1'):
             if self.request_version != 'HTTP/0.9':
-                self.wfile.write(("%s %d %s\r\n" %
+                self.wfile.write(('%s %d %s\r\n' %
                     (self.protocol_version, 100, 'Continue')).encode('latin1', 'strict'))
                 self.end_headers()
         return result
 
     def do_GET(self):
         self.method = requests.get
-        content_length = self.headers.get('Content-Length')
-        if content_length:
-            self.data_bytes = self.rfile.read(int(content_length))
+        self.read_content()
         self.forward('GET')
 
     def do_PUT(self):
-        self.data_bytes = self.rfile.read(int(self.headers['Content-Length']))
         self.method = requests.put
+        self.read_content()
         self.forward('PUT')
 
     def do_POST(self):
-        self.data_bytes = self.rfile.read(int(self.headers['Content-Length']))
         self.method = requests.post
+        self.read_content()
         self.forward('POST')
 
     def do_DELETE(self):
+        self.data_bytes = None
         self.method = requests.delete
         self.forward('DELETE')
 
     def do_HEAD(self):
+        self.data_bytes = None
         self.method = requests.head
         self.forward('HEAD')
 
     def do_PATCH(self):
         self.method = requests.patch
-        self.data_bytes = self.rfile.read(int(self.headers['Content-Length']))
+        self.read_content()
         self.forward('PATCH')
 
     def do_OPTIONS(self):
+        self.data_bytes = None
         self.method = requests.options
         self.forward('OPTIONS')
 
+    def read_content(self):
+        content_length = self.headers.get('Content-Length')
+        if content_length:
+            self.data_bytes = self.rfile.read(int(content_length))
+            return
+
+        self.data_bytes = None
+        if self.method in (requests.post, requests.put):
+            LOG.warning('Expected Content-Length header not found in POST/PUT request')
+
+            # If the Content-Length header is missing, try to read
+            # content from the socket using a socket timeout.
+            socket_timeout_secs = 0.5
+            self.request.settimeout(socket_timeout_secs)
+            block_length = 1
+            while True:
+                try:
+                    # TODO find a more efficient way to do this!
+                    tmp = self.rfile.read(block_length)
+                    if self.data_bytes is None:
+                        self.data_bytes = tmp
+                    else:
+                        self.data_bytes += tmp
+                except socket.timeout:
+                    break
+
+    def build_x_forwarded_for(self, headers):
+        x_forwarded_for = headers.get('X-Forwarded-For')
+
+        client_address = self.client_address[0]
+        server_address = ':'.join(map(str, self.server.server_address))
+
+        if x_forwarded_for:
+            x_forwarded_for_list = (x_forwarded_for, client_address, server_address)
+        else:
+            x_forwarded_for_list = (client_address, server_address)
+
+        return ', '.join(x_forwarded_for_list)
+
     def forward(self, method):
         path = self.path
         if '://' in path:
             path = '/' + path.split('://', 1)[1].split('/', 1)[1]
-        proxy_url = 'http://%s%s' % (self.proxy.forward_host, path)
+        proxy_url = '%s%s' % (self.proxy.forward_url, path)
         target_url = self.path
         if '://' not in target_url:
-            target_url = 'http://%s%s' % (self.proxy.forward_host, target_url)
-        data = None
-        if method in ['POST', 'PUT', 'PATCH']:
-            data_string = self.data_bytes
-            try:
-                if not isinstance(data_string, string_types):
-                    data_string = data_string.decode(DEFAULT_ENCODING)
-                data = json.loads(data_string)
-            except Exception as e:
-                # unable to parse JSON, fallback to verbatim string/bytes
-                data = data_string
+            target_url = '%s%s' % (self.proxy.forward_url, target_url)
+        data = self.data_bytes
 
         forward_headers = CaseInsensitiveDict(self.headers)
         # update original "Host" header (moto s3 relies on this behavior)
@@ -157,36 +206,71 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
         if 'localhost.atlassian.io' in forward_headers.get('Host'):
             forward_headers['host'] = 'localhost'
 
+        forward_headers['X-Forwarded-For'] = self.build_x_forwarded_for(forward_headers)
+
+        # force close connection
+        if forward_headers.get('Connection', '').lower() != 'keep-alive':
+            self.close_connection = 1
+
         try:
             response = None
             modified_request = None
             # update listener (pre-invocation)
-            if self.proxy.update_listener:
-                listener_result = self.proxy.update_listener.forward_request(method=method,
+            for listener in self.DEFAULT_LISTENERS + [self.proxy.update_listener]:
+                if not listener:
+                    continue
+                listener_result = listener.forward_request(method=method,
                     path=path, data=data, headers=forward_headers)
                 if isinstance(listener_result, Response):
                     response = listener_result
+                    break
+                if isinstance(listener_result, dict):
+                    response = Response()
+                    response._content = json.dumps(listener_result)
+                    response.status_code = 200
+                    break
                 elif isinstance(listener_result, Request):
                     modified_request = listener_result
                     data = modified_request.data
                     forward_headers = modified_request.headers
+                    break
                 elif listener_result is not True:
                     # get status code from response, or use Bad Gateway status code
                     code = listener_result if isinstance(listener_result, int) else 503
                     self.send_response(code)
+                    self.send_header('Content-Length', '0')
                     self.end_headers()
                     return
+            # perform the actual invocation of the backend service
             if response is None:
+                forward_headers['Connection'] = forward_headers.get('Connection') or 'close'
+                data_to_send = self.data_bytes
+                request_url = proxy_url
                 if modified_request:
-                    response = self.method(proxy_url, data=modified_request.data,
-                        headers=modified_request.headers)
-                else:
-                    response = self.method(proxy_url, data=self.data_bytes,
-                        headers=forward_headers)
+                    if modified_request.url:
+                        request_url = '%s%s' % (self.proxy.forward_url, modified_request.url)
+                    data_to_send = modified_request.data
+
+                response = self.method(request_url, data=data_to_send,
+                    headers=forward_headers, stream=True)
+
+                # prevent requests from processing response body
+                if not response._content_consumed and response.raw:
+                    response._content = response.raw.read()
             # update listener (post-invocation)
             if self.proxy.update_listener:
-                updated_response = self.proxy.update_listener.return_response(method=method,
-                    path=path, data=data, headers=forward_headers, response=response)
+                kwargs = {
+                    'method': method,
+                    'path': path,
+                    'data': data,
+                    'headers': forward_headers,
+                    'response': response
+                }
+                if 'request_handler' in inspect.getargspec(self.proxy.update_listener.return_response)[0]:
+                    # some listeners (e.g., sqs_listener.py) require additional details like the original
+                    # request port, hence we pass in a reference to this request handler as well.
+                    kwargs['request_handler'] = self
+                updated_response = self.proxy.update_listener.return_response(**kwargs)
                 if isinstance(updated_response, Response):
                     response = updated_response
 
@@ -195,8 +279,10 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
 
             content_length_sent = False
             for header_key, header_value in iteritems(response.headers):
-                self.send_header(header_key, header_value)
-                content_length_sent = content_length_sent or header_key.lower() == 'content-length'
+                # filter out certain headers that we don't want to transmit
+                if header_key.lower() not in ('transfer-encoding', 'date', 'server'):
+                    self.send_header(header_key, header_value)
+                    content_length_sent = content_length_sent or header_key.lower() == 'content-length'
             if not content_length_sent:
                 self.send_header('Content-Length', '%s' % len(response.content) if response.content else 0)
 
@@ -207,31 +293,53 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
                 self.send_header('Access-Control-Allow-Methods', ','.join(CORS_ALLOWED_METHODS))
             if 'Access-Control-Allow-Headers' not in response.headers:
                 self.send_header('Access-Control-Allow-Headers', ','.join(CORS_ALLOWED_HEADERS))
+            if 'Access-Control-Expose-Headers' not in response.headers:
+                self.send_header('Access-Control-Expose-Headers', ','.join(CORS_EXPOSE_HEADERS))
 
             self.end_headers()
             if response.content and len(response.content):
-                self.wfile.write(bytes_(response.content))
-            self.wfile.flush()
+                self.wfile.write(to_bytes(response.content))
         except Exception as e:
             trace = str(traceback.format_exc())
-            conn_error = 'ConnectionRefusedError' in trace or 'NewConnectionError' in trace
-            if not self.proxy.quiet or not conn_error:
-                LOGGER.error("Error forwarding request: %s %s" % (e, trace))
+            conn_errors = ('ConnectionRefusedError', 'NewConnectionError',
+                           'Connection aborted', 'Unexpected EOF', 'Connection reset by peer')
+            conn_error = any(e in trace for e in conn_errors)
+            error_msg = 'Error forwarding request: %s %s' % (e, trace)
+            if 'Broken pipe' in trace:
+                LOG.warn('Connection prematurely closed by client (broken pipe).')
+            elif not self.proxy.quiet or not conn_error:
+                LOG.error(error_msg)
+                if os.environ.get(ENV_INTERNAL_TEST_RUN):
+                    # During a test run, we also want to print error messages, because
+                    # log messages are delayed until the entire test run is over, and
+                    # hence we are missing messages if the test hangs for some reason.
+                    print('ERROR: %s' % error_msg)
             self.send_response(502)  # bad gateway
             self.end_headers()
+            # force close connection
+            self.close_connection = 1
+        finally:
+            try:
+                self.wfile.flush()
+            except Exception as e:
+                LOG.warning('Unable to flush write file: %s' % e)
 
     def log_message(self, format, *args):
         return
 
 
 class GenericProxy(FuncThread):
-    def __init__(self, port, forward_host=None, ssl=False, host=None, update_listener=None, quiet=False, params={}):
+    def __init__(self, port, forward_url=None, ssl=False, host=None, update_listener=None, quiet=False, params={}):
         FuncThread.__init__(self, self.run_cmd, params, quiet=quiet)
         self.httpd = None
         self.port = port
         self.ssl = ssl
         self.quiet = quiet
-        self.forward_host = forward_host
+        if forward_url:
+            if '://' not in forward_url:
+                forward_url = 'http://%s' % forward_url
+            forward_url = forward_url.rstrip('/')
+        self.forward_url = forward_url
         self.update_listener = update_listener
         self.server_stopped = False
         # Required to enable 'Connection: keep-alive' for S3 uploads
@@ -250,7 +358,7 @@ class GenericProxy(FuncThread):
             self.httpd.serve_forever()
         except Exception as e:
             if not self.quiet or not self.server_stopped:
-                LOGGER.error('Exception running proxy on port %s: %s' % (self.port, traceback.format_exc()))
+                LOG.error('Exception running proxy on port %s: %s %s' % (self.port, e, traceback.format_exc()))
 
     def stop(self, quiet=False):
         self.quiet = quiet
@@ -274,10 +382,15 @@ def serve_flask_app(app, port, quiet=True, host=None, cors=True):
     if cors:
         CORS(app)
     if quiet:
-        log = logging.getLogger('werkzeug')
-        log.setLevel(logging.ERROR)
+        logging.getLogger('werkzeug').setLevel(logging.ERROR)
     if not host:
         host = '0.0.0.0'
     ssl_context = GenericProxy.get_flask_ssl_context()
+    app.config['ENV'] = 'development'
+
+    def noecho(*args, **kwargs):
+        pass
+
+    click.echo = noecho
     app.run(port=int(port), threaded=True, host=host, ssl_context=ssl_context)
     return app
