@@ -3,7 +3,7 @@ import json
 import time
 from six.moves import queue
 from localstack.config import TMP_FOLDER, CONFIG_FILE_PATH
-from localstack.constants import API_ENDPOINT, ENV_INTERNAL_TEST_RUN
+from localstack.constants import ENV_INTERNAL_TEST_RUN, API_ENDPOINT
 from localstack.utils.common import (JsonObject, to_str,
     timestamp, short_uid, save_file, FuncThread, load_file)
 from localstack.utils.common import safe_requests as requests
@@ -18,10 +18,17 @@ EVENT_KINESIS_CREATE_STREAM = 'kns.cs'
 EVENT_KINESIS_DELETE_STREAM = 'kns.ds'
 EVENT_LAMBDA_CREATE_FUNC = 'lmb.cf'
 EVENT_LAMBDA_DELETE_FUNC = 'lmb.df'
+EVENT_LAMBDA_INVOKE_FUNC = 'lmb.if'
 EVENT_SQS_CREATE_QUEUE = 'sqs.cq'
 EVENT_SQS_DELETE_QUEUE = 'sqs.dq'
+EVENT_SNS_CREATE_TOPIC = 'sns.ct'
+EVENT_SNS_DELETE_TOPIC = 'sns.dt'
 EVENT_S3_CREATE_BUCKET = 's3.cb'
 EVENT_S3_DELETE_BUCKET = 's3.db'
+EVENT_STEPFUNCTIONS_CREATE_SM = 'stf.cm'
+EVENT_STEPFUNCTIONS_DELETE_SM = 'stf.dm'
+EVENT_APIGW_CREATE_API = 'agw.ca'
+EVENT_APIGW_DELETE_API = 'agw.da'
 EVENT_DYNAMODB_CREATE_TABLE = 'ddb.ct'
 EVENT_DYNAMODB_DELETE_TABLE = 'ddb.dt'
 
@@ -36,8 +43,9 @@ class AnalyticsEvent(JsonObject):
         self.t = kwargs.get('timestamp') or kwargs.get('t') or timestamp()
         self.m_id = kwargs.get('machine_id') or kwargs.get('m_id') or get_machine_id()
         self.p_id = kwargs.get('process_id') or kwargs.get('p_id') or get_process_id()
-        self.e_t = kwargs.get('event_type') or kwargs.get('e_t')
         self.p = kwargs.get('payload') if kwargs.get('payload') is not None else kwargs.get('p')
+        self.k = kwargs.get('api_key') or kwargs.get('k') or read_api_key_safe()
+        self.e_t = kwargs.get('event_type') or kwargs.get('e_t')
 
     def timestamp(self):
         return self.t
@@ -54,6 +62,17 @@ class AnalyticsEvent(JsonObject):
     def payload(self):
         return self.p
 
+    def api_key(self):
+        return self.k
+
+
+def read_api_key_safe():
+    try:
+        from localstack_ext.bootstrap.licensing import read_api_key
+        return read_api_key()
+    except Exception:
+        return None
+
 
 def get_or_create_file(config_file):
     if os.path.exists(config_file):
@@ -61,7 +80,7 @@ def get_or_create_file(config_file):
     try:
         save_file(config_file, '{}')
         return config_file
-    except Exception as e:
+    except Exception:
         pass
 
 
@@ -116,9 +135,9 @@ def poll_and_send_messages(params):
         try:
             event = EVENT_QUEUE.get(block=True, timeout=None)
             event = event.to_dict()
-            endpoint = '%s/events' % API_ENDPOINT
-            result = requests.post(endpoint, json=event)
-        except Exception as e:
+            endpoint = '%s/events' % API_ENDPOINT.rstrip('/')
+            requests.post(endpoint, json=event)
+        except Exception:
             # silently fail, make collection of usage data as non-intrusive as possible
             time.sleep(1)
 
@@ -141,6 +160,10 @@ def fire_event(event_type, payload=None):
     if not SENDER_THREAD:
         SENDER_THREAD = FuncThread(poll_and_send_messages, {})
         SENDER_THREAD.start()
+    api_key = read_api_key_safe()
+    if not api_key:
+        # only store events if API key has been specified
+        return
     if payload is None:
         payload = {}
     if isinstance(payload, dict):
@@ -149,5 +172,5 @@ def fire_event(event_type, payload=None):
         if os.environ.get(ENV_INTERNAL_TEST_RUN):
             payload['int'] = True
 
-    event = AnalyticsEvent(event_type=event_type, payload=payload)
+    event = AnalyticsEvent(event_type=event_type, payload=payload, api_key=api_key)
     EVENT_QUEUE.put_nowait(event)
