@@ -1,15 +1,20 @@
 import os
-import boto3
+import re
 import json
+import time
+import boto3
 import base64
 import logging
-import re
-from six import iteritems
-from threading import Timer
+import six
 from localstack import config
-from localstack.constants import *
-from localstack.utils.common import *
-from localstack.utils.aws.aws_models import *
+from localstack.constants import (
+    REGION_LOCAL, DEFAULT_REGION, LOCALHOST, MOTO_ACCOUNT_ID,
+    ENV_DEV, APPLICATION_AMZ_JSON_1_1, APPLICATION_AMZ_JSON_1_0,
+    APPLICATION_X_WWW_FORM_URLENCODED, TEST_AWS_ACCOUNT_ID)
+from localstack.utils.common import (
+    run_safe, to_str, is_string, is_string_or_bytes, make_http_request,
+    timestamp, is_port_open, get_service_protocol)
+from localstack.utils.aws.aws_models import KinesisStream
 
 # AWS environment variable names
 ENV_ACCESS_KEY = 'AWS_ACCESS_KEY_ID'
@@ -17,7 +22,10 @@ ENV_SECRET_KEY = 'AWS_SECRET_ACCESS_KEY'
 ENV_SESSION_TOKEN = 'AWS_SESSION_TOKEN'
 
 # set up logger
-LOGGER = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
+
+# cache local region
+LOCAL_REGION = None
 
 # Use this field if you want to provide a custom boto3 session.
 # This field takes priority over CREATE_NEW_SESSION_PER_BOTO3_CONNECTION
@@ -29,6 +37,9 @@ CREATE_NEW_SESSION_PER_BOTO3_CONNECTION = False
 # Used in AWS assume role function
 INITIAL_BOTO3_SESSION = None
 
+# Boto clients cache
+BOTO_CLIENTS_CACHE = {}
+
 # Assume role loop seconds
 DEFAULT_TIMER_LOOP_SECONDS = 60 * 50
 
@@ -37,7 +48,7 @@ class Environment(object):
     def __init__(self, region=None, prefix=None):
         # target is the runtime environment to use, e.g.,
         # 'local' for local mode
-        self.region = region or DEFAULT_REGION
+        self.region = region or get_local_region()
         # prefix can be 'prod', 'stg', 'uat-1', etc.
         self.prefix = prefix
 
@@ -52,7 +63,7 @@ class Environment(object):
         if len(parts) == 1:
             if s in PREDEFINED_ENVIRONMENTS:
                 return PREDEFINED_ENVIRONMENTS[s]
-            parts = [DEFAULT_REGION, s]
+            parts = [get_local_region(), s]
         if len(parts) > 2:
             raise Exception('Invalid environment string "%s"' % s)
         region = parts[0]
@@ -106,6 +117,10 @@ def get_environment(env=None, region_name=None):
     return env
 
 
+def is_local_env(env):
+    return not env or env.region == REGION_LOCAL or env.prefix == ENV_DEV
+
+
 def connect_to_resource(service_name, env=None, region_name=None, endpoint_url=None):
     """
     Generic method to obtain an AWS service resource using boto3, based on environment, region, or custom endpoint_url.
@@ -114,13 +129,15 @@ def connect_to_resource(service_name, env=None, region_name=None, endpoint_url=N
 
 
 def get_boto3_credentials():
+    global INITIAL_BOTO3_SESSION
     if CUSTOM_BOTO3_SESSION:
         return CUSTOM_BOTO3_SESSION.get_credentials()
-    return boto3.session.Session().get_credentials()
+    if not INITIAL_BOTO3_SESSION:
+        INITIAL_BOTO3_SESSION = boto3.session.Session()
+    return INITIAL_BOTO3_SESSION.get_credentials()
 
 
 def get_boto3_session():
-    my_session = None
     if CUSTOM_BOTO3_SESSION:
         return CUSTOM_BOTO3_SESSION
     if CREATE_NEW_SESSION_PER_BOTO3_CONNECTION:
@@ -129,26 +146,55 @@ def get_boto3_session():
     return boto3
 
 
-def get_local_service_url(service_name):
+def get_local_region():
+    global LOCAL_REGION
+    if LOCAL_REGION is None:
+        session = boto3.session.Session()
+        LOCAL_REGION = session.region_name or ''
+    return LOCAL_REGION or DEFAULT_REGION
+
+
+def get_local_service_url(service_name_or_port):
+    """ Return the local service URL for the given service name or port. """
+    if isinstance(service_name_or_port, int):
+        return '%s://%s:%s' % (get_service_protocol(), LOCALHOST, service_name_or_port)
+    service_name = service_name_or_port
     if service_name == 's3api':
         service_name = 's3'
     return os.environ['TEST_%s_URL' % (service_name.upper().replace('-', '_'))]
 
 
-def connect_to_service(service_name, client=True, env=None, region_name=None, endpoint_url=None):
+def is_service_enabled(service_name):
+    """ Return whether the service with the given name (e.g., "lambda") is available. """
+    try:
+        url = get_local_service_url(service_name)
+        assert url
+        return is_port_open(url, http_path='/', expect_success=False)
+    except Exception:
+        return False
+
+
+def connect_to_service(service_name, client=True, env=None, region_name=None, endpoint_url=None, config=None):
     """
     Generic method to obtain an AWS service client using boto3, based on environment, region, or custom endpoint_url.
     """
-    env = get_environment(env, region_name=region_name)
-    my_session = get_boto3_session()
-    method = my_session.client if client else my_session.resource
-    verify = True
-    if not endpoint_url:
-        if env.region == REGION_LOCAL:
-            endpoint_url = get_local_service_url(service_name)
-            verify = False
-    region = env.region if env.region != REGION_LOCAL else DEFAULT_REGION
-    return method(service_name, region_name=region, endpoint_url=endpoint_url, verify=verify)
+    key_elements = [service_name, client, env, region_name, endpoint_url, config]
+    cache_key = '/'.join([str(k) for k in key_elements])
+    if cache_key not in BOTO_CLIENTS_CACHE:
+        # Cache clients, as this is a relatively expensive operation
+        env = get_environment(env, region_name=region_name)
+        my_session = get_boto3_session()
+        method = my_session.client if client else my_session.resource
+        verify = True
+        if not endpoint_url:
+            if is_local_env(env):
+                endpoint_url = get_local_service_url(service_name)
+                verify = False
+        region = env.region if env.region != REGION_LOCAL else get_local_region()
+        BOTO_CLIENTS_CACHE[cache_key] = method(service_name, region_name=region,
+            endpoint_url=endpoint_url, verify=verify, config=config)
+
+    return BOTO_CLIENTS_CACHE[cache_key]
 
 
 class VelocityInput:
@@ -198,6 +244,42 @@ def render_velocity_template(template, context, as_json=False):
     return replaced
 
 
+def check_valid_region(headers):
+    """ Check whether a valid region is provided, and if not then raise an Exception. """
+    auth_header = headers.get('Authorization')
+    if not auth_header:
+        raise Exception('Unable to find "Authorization" header in request')
+    replaced = re.sub(r'.*Credential=([^,]+),.*', r'\1', auth_header)
+    if auth_header == replaced:
+        raise Exception('Unable to find "Credential" section in "Authorization" header')
+    # Format is: <your-access-key-id>/<date>/<aws-region>/<aws-service>/aws4_request
+    # See https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-auth-using-authorization-header.html
+    parts = replaced.split('/')
+    region = parts[2]
+    if region not in config.VALID_REGIONS:
+        raise Exception('Invalid region specified in "Authorization" header: "%s"' % region)
+
+
+def fix_account_id_in_arns(response, colon_delimiter=':', existing=None, replace=None):
+    """ Fix the account ID in the ARNs returned in the given Flask response or string """
+    existing = existing or ['123456789', MOTO_ACCOUNT_ID]
+    existing = existing if isinstance(existing, list) else [existing]
+    replace = replace or TEST_AWS_ACCOUNT_ID
+    is_str_obj = is_string_or_bytes(response)
+    content = to_str(response if is_str_obj else response._content)
+
+    replace = r'arn{col}aws{col}\1{col}\2{col}{acc}{col}'.format(col=colon_delimiter, acc=replace)
+    for acc_id in existing:
+        regex = r'arn{col}aws{col}([^:%]+){col}([^:%]*){col}{acc}{col}'.format(col=colon_delimiter, acc=acc_id)
+        content = re.sub(regex, replace, content)
+
+    if not is_str_obj:
+        response._content = content
+        response.headers['content-length'] = len(response._content)
+        return response
+    return content
+
+
 def get_s3_client():
     return boto3.resource('s3',
         endpoint_url=config.TEST_S3_URL,
@@ -206,19 +288,35 @@ def get_s3_client():
         verify=False)
 
 
+def extract_region_from_auth_header(headers):
+    auth = headers.get('Authorization') or ''
+    region = re.sub(r'.*Credential=[^/]+/[^/]+/([^/]+)/.*', r'\1', auth)
+    region = region or get_local_region()
+    return region
+
+
+def extract_region_from_arn(arn):
+    parts = arn.split(':')
+    return parts[3] if len(parts) > 1 else None
+
+
 def get_account_id(account_id=None, env=None):
     if account_id:
         return account_id
     env = get_environment(env)
-    if env.region == REGION_LOCAL:
+    if is_local_env(env):
         return os.environ['TEST_AWS_ACCOUNT_ID']
-    raise Exception('Unable to determine AWS account ID')
+    raise Exception('Unable to determine AWS account ID (%s, %s)' % (account_id, env))
 
 
 def role_arn(role_name, account_id=None, env=None):
+    if not role_name:
+        return role_name
+    if role_name.startswith('arn:aws:iam::'):
+        return role_name
     env = get_environment(env)
     account_id = get_account_id(account_id, env=env)
-    return "arn:aws:iam::%s:role/%s" % (account_id, role_name)
+    return 'arn:aws:iam::%s:role/%s' % (account_id, role_name)
 
 
 def iam_resource_arn(resource, role=None, env=None):
@@ -235,60 +333,129 @@ def get_iam_role(resource, env=None):
 
 def dynamodb_table_arn(table_name, account_id=None):
     account_id = get_account_id(account_id)
-    return "arn:aws:dynamodb:%s:%s:table/%s" % (DEFAULT_REGION, account_id, table_name)
+    return 'arn:aws:dynamodb:%s:%s:table/%s' % (get_local_region(), account_id, table_name)
 
 
 def dynamodb_stream_arn(table_name, account_id=None):
     account_id = get_account_id(account_id)
-    return ("arn:aws:dynamodb:%s:%s:table/%s/stream/%s" %
-        (DEFAULT_REGION, account_id, table_name, timestamp()))
+    return ('arn:aws:dynamodb:%s:%s:table/%s/stream/%s' %
+        (get_local_region(), account_id, table_name, timestamp()))
 
 
 def lambda_function_arn(function_name, account_id=None):
-    pattern = 'arn:aws:lambda:.*:.*:function:.*'
+    pattern = 'arn:aws:lambda:.*:.*:(function|layer):.*'
     if re.match(pattern, function_name):
         return function_name
-    if len(function_name.split(':')) > 1:
+    if ':' in function_name:
         raise Exception('Lambda function name should not contain a colon ":"')
     account_id = get_account_id(account_id)
-    return pattern.replace('.*', '%s') % (DEFAULT_REGION, account_id, function_name)
+    pattern = re.sub(r'\([^\|]+\|.+\)', 'function', pattern)
+    return pattern.replace('.*', '%s') % (get_local_region(), account_id, function_name)
+
+
+def lambda_function_name(name_or_arn):
+    if ':' not in name_or_arn:
+        return name_or_arn
+    parts = name_or_arn.split(':')
+    # name is index #6 in pattern: arn:aws:lambda:.*:.*:function:.*
+    return parts[6]
+
+
+def state_machine_arn(name, account_id=None):
+    if ':' in name:
+        return name
+    account_id = get_account_id(account_id)
+    pattern = 'arn:aws:states:%s:%s:stateMachine:%s'
+    return pattern % (get_local_region(), account_id, name)
+
+
+def fix_arn(arn):
+    """ Function that attempts to "canonicalize" the given ARN. This includes converting
+        resource names to ARNs, replacing incorrect regions, account IDs, etc. """
+    if arn.startswith('arn:aws:lambda'):
+        return lambda_function_arn(lambda_function_name(arn))
+    LOG.warning('Unable to fix/canonicalize ARN: %s' % arn)
+    return arn
 
 
 def cognito_user_pool_arn(user_pool_id, account_id=None):
     account_id = get_account_id(account_id)
-    return 'arn:aws:cognito-idp:%s:%s:userpool/%s' % (DEFAULT_REGION, account_id, user_pool_id)
+    return 'arn:aws:cognito-idp:%s:%s:userpool/%s' % (get_local_region(), account_id, user_pool_id)
 
 
 def kinesis_stream_arn(stream_name, account_id=None):
     account_id = get_account_id(account_id)
-    return "arn:aws:kinesis:%s:%s:stream/%s" % (DEFAULT_REGION, account_id, stream_name)
+    return 'arn:aws:kinesis:%s:%s:stream/%s' % (get_local_region(), account_id, stream_name)
 
 
 def firehose_stream_arn(stream_name, account_id=None):
     account_id = get_account_id(account_id)
-    return ("arn:aws:firehose:%s:%s:deliverystream/%s" % (DEFAULT_REGION, account_id, stream_name))
+    return ('arn:aws:firehose:%s:%s:deliverystream/%s' % (get_local_region(), account_id, stream_name))
 
 
 def s3_bucket_arn(bucket_name, account_id=None):
-    return "arn:aws:s3:::%s" % (bucket_name)
+    return 'arn:aws:s3:::%s' % (bucket_name)
 
 
-def sqs_queue_arn(queue_name, account_id=None):
+def create_sqs_queue(queue_name, env=None):
+    env = get_environment(env)
+    # queue
+    conn = connect_to_service('sqs', env=env)
+    return conn.create_queue(QueueName=queue_name)
+
+
+def sqs_queue_arn(queue_name, account_id=None, region_name=None):
     account_id = get_account_id(account_id)
-    return ("arn:aws:sqs:%s:%s:%s" % (DEFAULT_REGION, account_id, queue_name))
+    region_name = region_name or DEFAULT_REGION
+    return ('arn:aws:sqs:%s:%s:%s' % (region_name, account_id, queue_name))
+
+
+def sqs_queue_name(queue_arn):
+    parts = queue_arn.split(':')
+    return queue_arn if len(parts) == 1 else parts[5]
 
 
 def sns_topic_arn(topic_name, account_id=None):
     account_id = get_account_id(account_id)
-    return ("arn:aws:sns:%s:%s:%s" % (DEFAULT_REGION, account_id, topic_name))
+    return ('arn:aws:sns:%s:%s:%s' % (get_local_region(), account_id, topic_name))
 
 
-def get_sqs_queue_url(queue_name):
-    client = connect_to_service('sqs')
+def get_sqs_queue_url(queue_arn):
+    region_name = extract_region_from_arn(queue_arn)
+    queue_name = sqs_queue_name(queue_arn)
+    client = connect_to_service('sqs', region_name=region_name)
     response = client.get_queue_url(QueueName=queue_name)
     return response['QueueUrl']
 
 
+def sqs_receive_message(queue_arn):
+    region_name = extract_region_from_arn(queue_arn)
+    client = connect_to_service('sqs', region_name=region_name)
+    queue_url = get_sqs_queue_url(queue_arn)
+    response = client.receive_message(QueueUrl=queue_url)
+    return response
+
+
+def mock_aws_request_headers(service='dynamodb', region_name=None):
+    ctype = APPLICATION_AMZ_JSON_1_0
+    if service == 'kinesis':
+        ctype = APPLICATION_AMZ_JSON_1_1
+    elif service == 'sqs':
+        ctype = APPLICATION_X_WWW_FORM_URLENCODED
+    access_key = get_boto3_credentials().access_key
+    region_name = region_name or DEFAULT_REGION
+    headers = {
+        'Content-Type': ctype,
+        'Accept-Encoding': 'identity',
+        'X-Amz-Date': '20160623T103251Z',
+        'Authorization': ('AWS4-HMAC-SHA256 ' +
+            'Credential=%s/20160623/%s/%s/aws4_request, ' +
+            'SignedHeaders=content-type;host;x-amz-date;x-amz-target, Signature=1234') % (
+                access_key, region_name, service)
+    }
+    return headers
+
+
 def dynamodb_get_item_raw(request):
     headers = mock_aws_request_headers()
     headers['X-Amz-Target'] = 'DynamoDB_20120810.GetItem'
@@ -298,20 +465,38 @@ def dynamodb_get_item_raw(request):
     return new_item
 
 
-def mock_aws_request_headers(service='dynamodb'):
-    ctype = APPLICATION_AMZ_JSON_1_0
-    if service == 'kinesis':
-        ctype = APPLICATION_AMZ_JSON_1_1
-    access_key = get_boto3_credentials().access_key
-    headers = {
-        'Content-Type': ctype,
-        'Accept-Encoding': 'identity',
-        'X-Amz-Date': '20160623T103251Z',
-        'Authorization': ('AWS4-HMAC-SHA256 ' +
-            'Credential=%s/20160623/us-east-1/%s/aws4_request, ' +
-            'SignedHeaders=content-type;host;x-amz-date;x-amz-target, Signature=1234') % (access_key, service)
-    }
-    return headers
+def create_dynamodb_table(table_name, partition_key, env=None, stream_view_type=None):
+    """Utility method to create a DynamoDB table"""
+
+    dynamodb = connect_to_service('dynamodb', env=env, client=True)
+    stream_spec = {'StreamEnabled': False}
+    key_schema = [{
+        'AttributeName': partition_key,
+        'KeyType': 'HASH'
+    }]
+    attr_defs = [{
+        'AttributeName': partition_key,
+        'AttributeType': 'S'
+    }]
+    if stream_view_type is not None:
+        stream_spec = {
+            'StreamEnabled': True,
+            'StreamViewType': stream_view_type
+        }
+    table = None
+    try:
+        table = dynamodb.create_table(TableName=table_name, KeySchema=key_schema,
+            AttributeDefinitions=attr_defs, ProvisionedThroughput={
+                'ReadCapacityUnits': 10, 'WriteCapacityUnits': 10
+            },
+            StreamSpecification=stream_spec
+        )
+    except Exception as e:
+        if 'ResourceInUseException' in str(e):
+            # Table already exists -> return table reference
+            return connect_to_resource('dynamodb', env=env).Table(table_name)
+    time.sleep(2)
+    return table
 
 
 def get_apigateway_integration(api_id, method, path, env=None):
@@ -345,10 +530,10 @@ def get_apigateway_resource_for_path(api_id, path, parent=None, resources=None):
     return None
 
 
-def get_apigateway_path_for_resource(api_id, resource_id, path_suffix='', resources=None):
+def get_apigateway_path_for_resource(api_id, resource_id, path_suffix='', resources=None, region_name=None):
     if resources is None:
-        apigateway = connect_to_service(service_name='apigateway')
-        resources = apigateway.get_resources(restApiId=api_id, limit=100)
+        apigateway = connect_to_service(service_name='apigateway', region_name=region_name)
+        resources = apigateway.get_resources(restApiId=api_id, limit=100)['items']
     target_resource = list(filter(lambda res: res['id'] == resource_id, resources))[0]
     path_part = target_resource.get('pathPart', '')
     if path_suffix:
@@ -359,12 +544,13 @@ def get_apigateway_path_for_resource(api_id, resource_id, path_suffix='', resour
     parent_id = target_resource.get('parentId')
     if not parent_id:
         return '/%s' % path_suffix
-    return get_apigateway_path_for_resource(api_id, parent_id, path_suffix=path_suffix, resources=resources)
+    return get_apigateway_path_for_resource(api_id, parent_id,
+        path_suffix=path_suffix, resources=resources, region_name=region_name)
 
 
 def create_api_gateway(name, description=None, resources=None, stage_name=None,
-        enabled_api_keys=[], env=None, usage_plan_name=None):
-    client = connect_to_service('apigateway', env=env)
+        enabled_api_keys=[], env=None, usage_plan_name=None, region_name=None):
+    client = connect_to_service('apigateway', env=env, region_name=region_name)
     if not resources:
         resources = []
     if not stage_name:
@@ -374,20 +560,22 @@ def create_api_gateway(name, description=None, resources=None, stage_name=None,
     if not description:
         description = 'Test description for API "%s"' % name
 
-    LOGGER.info('Creating API resources under API Gateway "%s".' % name)
+    LOG.info('Creating API resources under API Gateway "%s".' % name)
     api = client.create_rest_api(name=name, description=description)
     # list resources
     api_id = api['id']
     resources_list = client.get_resources(restApiId=api_id)
     root_res_id = resources_list['items'][0]['id']
     # add API resources and methods
-    for path, methods in iteritems(resources):
-        if '/' in path:
-            raise Exception('Currently only works for root-level resources.')
-        api_resource = client.create_resource(restApiId=api_id, parentId=root_res_id, pathPart=path)
+    for path, methods in six.iteritems(resources):
+        # create resources recursively
+        parent_id = root_res_id
+        for path_part in path.split('/'):
+            api_resource = client.create_resource(restApiId=api_id, parentId=parent_id, pathPart=path_part)
+            parent_id = api_resource['id']
         # add methods to the API resource
         for method in methods:
-            api_method = client.put_method(
+            client.put_method(
                 restApiId=api_id,
                 resourceId=api_resource['id'],
                 httpMethod=method['httpMethod'],
@@ -396,14 +584,16 @@ def create_api_gateway(name, description=None, resources=None, stage_name=None,
             )
             # create integrations for this API resource/method
             integrations = method['integrations']
-            create_api_gateway_integrations(api_id, api_resource['id'], method, integrations, env=env)
+            create_api_gateway_integrations(api_id, api_resource['id'], method,
+                integrations, env=env, region_name=region_name)
     # deploy the API gateway
-    api_deployed = client.create_deployment(restApiId=api_id, stageName=stage_name)
+    client.create_deployment(restApiId=api_id, stageName=stage_name)
     return api
 
 
-def create_api_gateway_integrations(api_id, resource_id, method, integrations=[], env=None):
-    client = connect_to_service('apigateway', env=env)
+def create_api_gateway_integrations(api_id, resource_id, method,
+        integrations=[], env=None, region_name=None):
+    client = connect_to_service('apigateway', env=env, region_name=region_name)
     for integration in integrations:
         req_templates = integration.get('requestTemplates') or {}
         res_templates = integration.get('responseTemplates') or {}
@@ -411,7 +601,7 @@ def create_api_gateway_integrations(api_id, resource_id, method, integrations=[]
         client_error_code = integration.get('clientErrorCode') or '400'
         server_error_code = integration.get('serverErrorCode') or '500'
         # create integration
-        response = client.put_integration(
+        client.put_integration(
             restApiId=api_id,
             resourceId=resource_id,
             httpMethod=method['httpMethod'],
@@ -428,7 +618,7 @@ def create_api_gateway_integrations(api_id, resource_id, method, integrations=[]
         # create response configs
         for response_config in response_configs:
             # create integration response
-            response = client.put_integration_response(
+            client.put_integration_response(
                 restApiId=api_id,
                 resourceId=resource_id,
                 httpMethod=method['httpMethod'],
@@ -437,7 +627,7 @@ def create_api_gateway_integrations(api_id, resource_id, method, integrations=[]
                 selectionPattern=response_config['pattern']
             )
             # create method response
-            response = client.put_method_response(
+            client.put_method_response(
                 restApiId=api_id,
                 resourceId=resource_id,
                 httpMethod=method['httpMethod'],
@@ -445,9 +635,13 @@ def create_api_gateway_integrations(api_id, resource_id, method, integrations=[]
             )
 
 
+def apigateway_invocations_arn(lambda_uri):
+    return 'arn:aws:apigateway:%s:lambda:path/2015-03-31/functions/%s/invocations' % (DEFAULT_REGION, lambda_uri)
+
+
 def get_elasticsearch_endpoint(domain=None, region_name=None):
     env = get_environment(region_name=region_name)
-    if env.region == REGION_LOCAL:
+    if is_local_env(env):
         return os.environ['TEST_ELASTICSEARCH_URL']
     # get endpoint from API
     es_client = connect_to_service(service_name='es', region_name=env.region)
@@ -463,14 +657,14 @@ def connect_elasticsearch(endpoint=None, domain=None, region_name=None, env=None
     env = get_environment(env, region_name=region_name)
     verify_certs = False
     use_ssl = False
-    if not endpoint and env.region == REGION_LOCAL:
+    if not endpoint and is_local_env(env):
         endpoint = os.environ['TEST_ELASTICSEARCH_URL']
-    if not endpoint and env.region != REGION_LOCAL and domain:
+    if not endpoint and not is_local_env(env) and domain:
         endpoint = get_elasticsearch_endpoint(domain=domain, region_name=env.region)
     # use ssl?
     if 'https://' in endpoint:
         use_ssl = True
-        if env.region != REGION_LOCAL:
+        if not is_local_env(env):
             verify_certs = True
 
     if CUSTOM_BOTO3_SESSION or (ENV_ACCESS_KEY in os.environ and ENV_SECRET_KEY in os.environ):
@@ -514,7 +708,7 @@ def kinesis_get_latest_records(stream_name, shard_id, count=10, env=None):
         for record in records:
             try:
                 record['Data'] = to_str(record['Data'])
-            except Exception as e:
+            except Exception:
                 pass
         result.extend(records)
         shard_iterator = records_response['NextShardIterator'] if records else False
