@@ -1,9 +1,9 @@
-import json
-from datetime import datetime, timedelta
-from flask import Flask, Response
+from datetime import datetime
+from flask import Response
 from localstack import config
-from localstack.utils.common import now_utc, make_http_request, to_str
 from localstack.utils.aws import aws_stack
+from localstack.utils.common import now_utc
+from localstack.utils.analytics import event_publisher
 
 
 # ---------------
@@ -11,9 +11,7 @@ from localstack.utils.aws import aws_stack
 # ---------------
 
 def dimension_lambda(kwargs):
-    func_name = kwargs.get('func_name')
-    if not func_name:
-        func_name = kwargs.get('func_arn').split(':function:')[1].split(':')[0]
+    func_name = _func_name(kwargs)
     return [{
         'Name': 'FunctionName',
         'Value': func_name
@@ -56,42 +54,29 @@ def publish_lambda_result(time_before, result, kwargs):
 # ---------------
 
 
-# TODO: this is a backdoor based hack until get_metric_statistics becomes available in moto
-def get_metric_statistics(Namespace, MetricName, Dimensions,
-        Period=60, StartTime=None, EndTime=None, Statistics=None):
-    if not StartTime:
-        StartTime = datetime.now() - timedelta(minutes=5)
-    if not EndTime:
-        EndTime = datetime.now()
-    if Statistics is None:
-        Statistics = ['Sum']
-    cloudwatch_url = aws_stack.get_local_service_url('cloudwatch')
-    url = '%s/?Action=GetMetricValues' % cloudwatch_url
-    all_metrics = make_http_request(url)
-    assert all_metrics.status_code == 200
-    datapoints = []
-    for datapoint in json.loads(to_str(all_metrics.content)):
-        if datapoint['Namespace'] == Namespace and datapoint['Name'] == MetricName:
-            dp_dimensions = datapoint['Dimensions']
-            all_present = all(m in dp_dimensions for m in Dimensions)
-            no_additional = all(m in Dimensions for m in dp_dimensions)
-            if all_present and no_additional:
-                datapoints.append(datapoint)
-    result = {
-        'Label': '%s/%s' % (Namespace, MetricName),
-        'Datapoints': datapoints
-    }
-    return result
+def _func_name(kwargs):
+    func_name = kwargs.get('func_name')
+    if not func_name:
+        func_name = kwargs.get('func_arn').split(':function:')[1].split(':')[0]
+    return func_name
+
+
+def publish_event(time_before, result, kwargs):
+    event_publisher.fire_event(
+        event_publisher.EVENT_LAMBDA_INVOKE_FUNC,
+        payload={'f': _func_name(kwargs), 'd': now_utc() - time_before, 'r': result[0]})
 
 
 def publish_result(ns, time_before, result, kwargs):
     if ns == 'lambda':
         publish_lambda_result(time_before, result, kwargs)
+        publish_event(time_before, 'success', kwargs)
 
 
 def publish_error(ns, time_before, e, kwargs):
     if ns == 'lambda':
         publish_lambda_error(time_before, kwargs)
+        publish_event(time_before, 'error', kwargs)
 
 
 def cloudwatched(ns):
@@ -99,7 +84,6 @@ def cloudwatched(ns):
     def wrapping(func):
         def wrapped(*args, **kwargs):
             time_before = now_utc()
-            result = None
             try:
                 result = func(*args, **kwargs)
                 publish_result(ns, time_before, result, kwargs)
@@ -107,7 +91,9 @@ def cloudwatched(ns):
                 publish_error(ns, time_before, e, kwargs)
                 raise e
             finally:
-                time_after = now_utc()
+                # TODO
+                # time_after = now_utc()
+                pass
             return result
         return wrapped
     return wrapping
