@@ -1,23 +1,25 @@
 #!/usr/bin/env python
 
 import os
-import sys
 import re
-import tempfile
-import time
-import threading
 import logging
+import tempfile
+import threading
+import subprocess32 as subprocess
 from six.moves import queue as Queue
 from six.moves.urllib.parse import urlparse
 from amazon_kclpy import kcl
-from localstack.constants import *
+from localstack import config
 from localstack.config import HOSTNAME, USE_SSL
-from localstack.utils.common import *
-from localstack.utils.kinesis import kclipy_helper
-from localstack.utils.kinesis.kinesis_util import EventFileReaderThread
-from localstack.utils.common import ShellCommandThread, FuncThread
+from localstack.constants import (
+    LOCALSTACK_VENV_FOLDER, LOCALSTACK_ROOT_FOLDER, DEFAULT_REGION)
 from localstack.utils.aws import aws_stack
+from localstack.utils.common import (
+    run, TMP_THREADS, TMP_FILES, save_file, now, retry, short_uid, to_str,
+    chmod_r, rm_rf, ShellCommandThread, FuncThread)
+from localstack.utils.kinesis import kclipy_helper
 from localstack.utils.aws.aws_models import KinesisStream
+from localstack.utils.kinesis.kinesis_util import EventFileReaderThread
 
 
 EVENTS_FILE_PATTERN = os.path.join(tempfile.gettempdir(), 'kclipy.*.fifo')
@@ -25,7 +27,7 @@ LOG_FILE_PATTERN = os.path.join(tempfile.gettempdir(), 'kclipy.*.log')
 DEFAULT_DDB_LEASE_TABLE_SUFFIX = '-kclapp'
 
 # define Java class names
-MULTI_LANG_DAEMON_CLASS = 'com.atlassian.KinesisStarter'
+MULTI_LANG_DAEMON_CLASS = 'cloud.localstack.KinesisStarter'
 
 # set up log levels
 logging.SEVERE = 60
@@ -117,9 +119,8 @@ class KinesisProcessorThread(ShellCommandThread):
         if not params['log_file']:
             params['log_file'] = '%s.log' % props_file
             TMP_FILES.append(params['log_file'])
-        # print(cmd)
         env = aws_stack.get_environment()
-        quiet = env.region == REGION_LOCAL
+        quiet = aws_stack.is_local_env(env)
         ShellCommandThread.__init__(self, cmd, outfile=params['log_file'], env_vars=env_vars, quiet=quiet)
 
     @staticmethod
@@ -130,6 +131,7 @@ class KinesisProcessorThread(ShellCommandThread):
 
 
 class OutputReaderThread(FuncThread):
+
     def __init__(self, params):
         FuncThread.__init__(self, self.start_reading, params)
         self.buffer = []
@@ -160,13 +162,13 @@ class OutputReaderThread(FuncThread):
 
     @classmethod
     def get_log_level_names(cls, min_level):
-        return [logging.getLevelName(l) for l in LOG_LEVELS if l >= min_level]
+        return [logging.getLevelName(lvl) for lvl in LOG_LEVELS if lvl >= min_level]
 
     def get_logger_for_level_in_log_line(self, line):
         level = self.log_level
-        for l in LOG_LEVELS:
-            if l >= level:
-                level_name = logging.getLevelName(l)
+        for lvl in LOG_LEVELS:
+            if lvl >= level:
+                level_name = logging.getLevelName(lvl)
                 if re.match(r'.*(%s):.*' % level_name, line):
                     return getattr(self.logger, level_name.lower())
         return None
@@ -198,13 +200,23 @@ class OutputReaderThread(FuncThread):
                     self.buffer = []
 
     def _tail(self, file):
+        p = subprocess.Popen(['tail', '-f', file], stdout=subprocess.PIPE)
+        while True:
+            line = p.stdout.readline()
+            if not line:
+                break
+            line = to_str(line)
+            yield line.replace('\n', '')
+
+    def _tail_native(self, file):
+        # deprecated
         with open(file) as f:
             while self.running:
                 line = f.readline()
-                if line:  # empty if at EOF
-                    yield line.replace('\n', '')
-                else:
-                    time.sleep(0.1)
+                if not line:
+                    # empty if at EOF (non-empty, including newline, if not at EOF)
+                    return
+                yield line.replace('\n', '')
 
     def stop(self, quiet=True):
         self._stop_event.set()
@@ -256,10 +268,10 @@ def get_stream_info(stream_name, log_file=None, shards=None, env=None, endpoint_
         'env_vars': env_vars
     }
     # set local connection
-    if env.region == REGION_LOCAL:
+    if aws_stack.is_local_env(env):
         stream_info['conn_kwargs'] = {
             'host': HOSTNAME,
-            'port': DEFAULT_PORT_KINESIS,
+            'port': config.PORT_KINESIS,
             'is_secure': bool(USE_SSL)
         }
     if endpoint_url:
@@ -273,7 +285,7 @@ def get_stream_info(stream_name, log_file=None, shards=None, env=None, endpoint_
 
 
 def start_kcl_client_process(stream_name, listener_script, log_file=None, env=None, configs={},
-        endpoint_url=None, ddb_lease_table_suffix=None, env_vars={},
+        endpoint_url=None, ddb_lease_table_suffix=None, env_vars={}, region_name=None,
         kcl_log_level=DEFAULT_KCL_LOG_LEVEL, log_subscribers=[]):
     env = aws_stack.get_environment(env)
     # decide which credentials provider to use
@@ -281,13 +293,13 @@ def start_kcl_client_process(stream_name, listener_script, log_file=None, env=No
     if (('AWS_ASSUME_ROLE_ARN' in os.environ or 'AWS_ASSUME_ROLE_ARN' in env_vars) and
             ('AWS_ASSUME_ROLE_SESSION_NAME' in os.environ or 'AWS_ASSUME_ROLE_SESSION_NAME' in env_vars)):
         # use special credentials provider that can assume IAM roles and handle temporary STS auth tokens
-        credentialsProvider = 'com.atlassian.DefaultSTSAssumeRoleSessionCredentialsProvider'
+        credentialsProvider = 'cloud.localstack.DefaultSTSAssumeRoleSessionCredentialsProvider'
         # pass through env variables to child process
         for var_name in ['AWS_ASSUME_ROLE_ARN', 'AWS_ASSUME_ROLE_SESSION_NAME',
                 'AWS_ACCESS_KEY_ID', 'AWS_SECRET_ACCESS_KEY', 'AWS_SESSION_TOKEN']:
             if var_name in os.environ and var_name not in env_vars:
                 env_vars[var_name] = os.environ[var_name]
-    if env.region == REGION_LOCAL:
+    if aws_stack.is_local_env(env):
         # need to disable CBOR protocol, enforce use of plain JSON,
         # see https://github.com/mhart/kinesalite/issues/31
         env_vars['AWS_CBOR_DISABLE'] = 'true'
@@ -312,9 +324,9 @@ def start_kcl_client_process(stream_name, listener_script, log_file=None, env=No
         'initialPositionInStream': 'LATEST'
     }
     # set parameters for local connection
-    if env.region == REGION_LOCAL:
-        kwargs['kinesisEndpoint'] = '%s:%s' % (HOSTNAME, DEFAULT_PORT_KINESIS)
-        kwargs['dynamodbEndpoint'] = '%s:%s' % (HOSTNAME, DEFAULT_PORT_DYNAMODB)
+    if aws_stack.is_local_env(env):
+        kwargs['kinesisEndpoint'] = '%s:%s' % (HOSTNAME, config.PORT_KINESIS)
+        kwargs['dynamodbEndpoint'] = '%s:%s' % (HOSTNAME, config.PORT_DYNAMODB)
         kwargs['kinesisProtocol'] = 'http%s' % ('s' if USE_SSL else '')
         kwargs['dynamodbProtocol'] = 'http%s' % ('s' if USE_SSL else '')
         kwargs['disableCertChecking'] = 'true'
@@ -322,7 +334,7 @@ def start_kcl_client_process(stream_name, listener_script, log_file=None, env=No
     # create config file
     kclipy_helper.create_config_file(config_file=props_file, executableName=listener_script,
         streamName=stream_name, applicationName=stream_info['app_name'],
-        credentialsProvider=credentialsProvider, **kwargs)
+        credentialsProvider=credentialsProvider, region_name=region_name, **kwargs)
     TMP_FILES.append(props_file)
     # start stream consumer
     stream = KinesisStream(id=stream_name, params=stream_info)
@@ -339,11 +351,11 @@ def generate_processor_script(events_file, log_file=None):
         log_file = 'None'
     content = """#!/usr/bin/env python
 import os, sys, glob, json, socket, time, logging, tempfile
-import subprocess32 as subprocess
 logging.basicConfig(level=logging.INFO)
 for path in glob.glob('%s/lib/python*/site-packages'):
     sys.path.insert(0, path)
 sys.path.insert(0, '%s')
+import subprocess32 as subprocess
 from localstack.config import DEFAULT_ENCODING
 from localstack.utils.kinesis import kinesis_connector
 from localstack.utils.common import timestamp
@@ -386,7 +398,7 @@ if __name__ == '__main__':
     kinesis_connector.KinesisProcessor.run_processor(log_file=log_file, processor_func=receive_msg)
     """ % (LOCALSTACK_VENV_FOLDER, LOCALSTACK_ROOT_FOLDER, events_file, log_file)
     save_file(script_file, content)
-    run('chmod +x %s' % script_file)
+    chmod_r(script_file, 0o755)
     TMP_FILES.append(script_file)
     return script_file
 
@@ -394,7 +406,7 @@ if __name__ == '__main__':
 def listen_to_kinesis(stream_name, listener_func=None, processor_script=None,
         events_file=None, endpoint_url=None, log_file=None, configs={}, env=None,
         ddb_lease_table_suffix=None, env_vars={}, kcl_log_level=DEFAULT_KCL_LOG_LEVEL,
-        log_subscribers=[], wait_until_started=False):
+        log_subscribers=[], wait_until_started=False, fh_d_stream=None, region_name=None):
     """
     High-level function that allows to subscribe to a Kinesis stream
     and receive events in a listener function. A KCL client process is
@@ -407,10 +419,10 @@ def listen_to_kinesis(stream_name, listener_func=None, processor_script=None,
     if not processor_script:
         processor_script = generate_processor_script(events_file, log_file=log_file)
 
-    run('rm -f %s' % events_file)
+    rm_rf(events_file)
     # start event reader thread (this process)
     ready_mutex = threading.Semaphore(0)
-    thread = EventFileReaderThread(events_file, listener_func, ready_mutex=ready_mutex)
+    thread = EventFileReaderThread(events_file, listener_func, ready_mutex=ready_mutex, fh_d_stream=fh_d_stream)
     thread.start()
     # Wait until the event reader thread is ready (to avoid 'Connection refused' error on the UNIX socket)
     ready_mutex.acquire()
@@ -425,18 +437,18 @@ def listen_to_kinesis(stream_name, listener_func=None, processor_script=None,
     process = start_kcl_client_process(stream_name, processor_script,
         endpoint_url=endpoint_url, log_file=log_file, configs=configs, env=env,
         ddb_lease_table_suffix=ddb_lease_table_suffix, env_vars=env_vars, kcl_log_level=kcl_log_level,
-        log_subscribers=log_subscribers)
+        log_subscribers=log_subscribers, region_name=region_name)
 
     if wait_until_started:
         # Wait at most 90 seconds for initialization. Note that creating the DDB table can take quite a bit
         try:
             listener.sync_init.get(block=True, timeout=90)
-        except Exception as e:
+        except Exception:
             raise Exception('Timeout when waiting for KCL initialization.')
         # wait at most 30 seconds for shard lease notification
         try:
             listener.sync_take_shard.get(block=True, timeout=30)
-        except Exception as e:
+        except Exception:
             # this merely means that there is no shard available to take. Do nothing.
             pass
 
