@@ -4,7 +4,7 @@ import base64
 import traceback
 import requests
 import logging
-from localstack.config import DATA_DIR, DEFAULT_ENCODING
+from localstack.config import DATA_DIR
 from localstack.utils.aws import aws_stack
 from localstack.utils.common import to_bytes, to_str
 
@@ -39,7 +39,7 @@ def record(api, method, path, data, headers):
     try:
         if isinstance(data, dict):
             data = json.dumps(data)
-        if data:
+        if data or data in [u'', b'']:
             try:
                 data = to_bytes(data)
             except Exception as e:
@@ -65,7 +65,7 @@ def replay_command(command):
         data = base64.b64decode(data)
     endpoint = aws_stack.get_local_service_url(command['a'])
     full_url = (endpoint[:-1] if endpoint.endswith('/') else endpoint) + command['p']
-    result = function(full_url, data=data, headers=command['h'])
+    result = function(full_url, data=data, headers=command['h'], verify=False)
     return result
 
 
